/*
 *    MotionServer.c
 *
 *    Copyright (C) 2008-2011 Yvan Roch <yvan.roch@gmail.com>
 *
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <string.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "MotionServer.h"

/* FILE name for the rcfile parser */
extern 	FILE *rcfilein;
/* These array are filled by the configuration file parser */
extern int servoEnableBuffer[];
extern int servoPositionBuffer[];
extern int servoOffsetBuffer[];
extern int servoMinBuffer[];
extern int servoMaxBuffer[];
extern char moduleFilePathBuffer[];
extern int moduleLoadBuffer;
char pBuffer[BUFFER_SIZE];
void main_loop(int hSocket){
	int i;
	int ret;
	char read_buffer;
	ssize_t count;

	while(1){
		for (i=0;i<BUFFER_SIZE;i++){
			pBuffer[i]=0;
		}
		i=0;
		/* End Of Line detection */
		while(read_buffer != '\n'){
			/* read one char from socket into buffer */
			count=read(hSocket,&read_buffer,1);
			/* Error on reading */
			if (count == -1){
				printf("Read error: %d\n",errno);
				return;
			}
			if (count == 0){
				printf("Client socket closed.\n");
				return;
			}
			pBuffer[i]=read_buffer;
			i++;
		}
		/* Reset while condition */
		read_buffer=0;
		/* Add Final 0*/
		pBuffer[i]=0;
		/* Prepare command perser with the string read from network */
		command_scan_string(pBuffer);
		ret=commandparse();
		if (ret!=0){
			write(hSocket,MOTION_SERVER_ERROR,strlen(MOTION_SERVER_ERROR));
		} else {
			write(hSocket,pBuffer,strlen(pBuffer));
		}
	}
}
int main(int argc, char* argv[]){
	struct hostent* pHostInfo;   /* holds info about a machine */
	struct sockaddr_in Address; /* Internet socket address stuct */
	int nAddressSize=sizeof(struct sockaddr_in);
	char pBuffer[BUFFER_SIZE];
	int nHostPort;
	char config_file_name[1024];
	int ret;
	/* Read the .MotionServerrc config file */
	rcfilein=fopen(".MotionServerrc","r");
	if (!rcfilein){
		sprintf(config_file_name,"%s/.MotionServerrc",getenv("HOME"));
		rcfilein=fopen(config_file_name,"r");
		if (!rcfilein){
			perror("Error: no config file .MotionServerrc found!!!");
			exit(1);
		}
	}
	/* Parse config file */
	ret=rcfileparse();
	fclose(rcfilein);
	if (ret!=0){
		fprintf(stderr, "Configuration file error!!!\n");
		exit(1);
	}
	prinfDefaultConfig();
	/* Load the kernel module if configured */
	if (moduleLoadBuffer){
		char cmd[1024];
		/* Unload if loaded */
		sprintf(cmd,"%s %s",RMMOD,moduleFilePathBuffer);
		printf("Executing: %s\n",cmd);
		ret=system(cmd);
		/* Loading */
		sprintf(cmd,"%s %s",INSMOD,moduleFilePathBuffer);
		printf("Executing: %s\n",cmd);
		ret=system(cmd);
		if (ret!=0){
			fprintf(stderr, "Execution %s error!!!\n",cmd);
			exit(1);
		}
	}

	nHostPort=MOTION_SERVER_TCP_PORT;
	printf("\nStarting server");

	printf("\nMaking socket");
	/* make a socket */
	hServerSocket=socket(AF_INET,SOCK_STREAM,0);

	if(hServerSocket == SOCKET_ERROR)
	{
		printf("\nCould not make a socket\n");
		return 0;
	}

	/* fill address struct */
	Address.sin_addr.s_addr=INADDR_ANY;
	Address.sin_port=htons(nHostPort);
	Address.sin_family=AF_INET;

	printf("\nBinding to port %d",nHostPort);

	/* bind to a port */
	if(bind(hServerSocket,(struct sockaddr*)&Address,sizeof(Address))
						== SOCKET_ERROR)
	{
		printf("\nCould not connect to host\n");
		return 0;
	}
	/*  get port number */
	getsockname( hServerSocket, (struct sockaddr *) &Address,(socklen_t *)&nAddressSize);
	printf("opened socket as fd (%d) on port (%d) for stream i/o\n",hServerSocket, ntohs(Address.sin_port) );

	printf("Server\n\
		  sin_family        = %d\n\
		  sin_addr.s_addr   = %d\n\
		  sin_port          = %d\n"
		  , Address.sin_family
		  , Address.sin_addr.s_addr
		  , ntohs(Address.sin_port)
		);

	printf("\nMaking a listen queue of %d elements",QUEUE_SIZE);
	/* establish listen queue */
	if(listen(hServerSocket,QUEUE_SIZE) == SOCKET_ERROR)
	{
		printf("\nCould not listen\n");
		return 0;
	}

	printf("\nWaiting for a connection\n");
	while(1){
		/* get the connected socket */
		hSocket=accept(hServerSocket,(struct sockaddr*)&Address,(socklen_t *)&nAddressSize);
		printf("\nGot a connection");
		strcpy(pBuffer,MOTION_SERVER_SIGNATURE);
		printf("\nSending \"%s\" to client\n",pBuffer);
		/* number returned by read() and write() is the number of bytes
		** read or written, with -1 being that an error occured
		** write what we received back to the server */
		write(hSocket,pBuffer,strlen(pBuffer)+1);
		/* Main loop: Waiting for client events /*/
		main_loop(hSocket);

		printf("\nClosing the client socket");
		/* close client socket */
		if(close(hSocket) == SOCKET_ERROR){
			printf("\nCould not close socket\n");
			return 0;
		}
	}

}

void prinfDefaultConfig(void){
	int i;
	printf("prinfDefaultConfig:\n");
	printf("moduleFilePathBuffer: %s\n",moduleFilePathBuffer);
	printf("moduleLoadBuffer: %d\n",moduleLoadBuffer);
	for (i=0;i<NBR_OF_MANAGED_SERVOS;i++){
		printf("servoPositionBuffer servo %d:%d\n",i,servoPositionBuffer[i]);
		printf("servoOffsetBuffer servo %d:%d\n",i,servoOffsetBuffer[i]);
		printf("servoMinBuffer servo %d:%d\n",i,servoMinBuffer[i]);
		printf("servoMaxBuffer servo %d:%d\n",i,servoMaxBuffer[i]);
	}
}
void setServoPosition(int servoID,int value){
	char operationFileName[1024];
	char data[16];
	int file;
	int ret;
	sprintf(operationFileName,"%s%s%d/%s",SERVO_DRIVER_SYSFS_BASE,SERVO_DRIVER_SERVO_FILE,servoID,SERVO_DRIVER_SERVO_POSITION_FILE);
	file=open(operationFileName,O_WRONLY);
	if(file==-1){
		fprintf(stderr,"MotionServer setServoPosition: failed to open %s",operationFileName);
		cleaning(errno);
	}
	/* Keep the value */
	servoPositionBuffer[servoID]=value;
	sprintf(data,"%d",value);
	ret=write(file,data,strlen(data));
	if(ret==-1){
		fprintf(stderr,"MotionServer setServoPosition: failed to write in %s",operationFileName);
		cleaning(errno);
	}
	close(file);
}
void setServoEnable(int servoID,int value){
	char operationFileName[1024];
	char data[16];
	int file;
	int ret;
	sprintf(operationFileName,"%s%s%d/%s",SERVO_DRIVER_SYSFS_BASE,SERVO_DRIVER_SERVO_FILE,servoID,SERVO_DRIVER_SERVO_ENABLE_FILE);
	file=open(operationFileName,O_WRONLY);
	if(file==-1){
		fprintf(stderr,"MotionServer setServoEnable: failed to open %s",operationFileName);
		cleaning(errno);
	}
	/* Keep the value */
	servoEnableBuffer[servoID]=value;
	sprintf(data,"%d",value);
	ret=write(file,data,strlen(data));
	if(ret==-1){
		fprintf(stderr,"MotionServer setServoEnable: failed to write in %s",operationFileName);
		cleaning(errno);
	}
	close(file);
}
void setServoOffset(int servoID,int value){
	printf("setServoOffset\n");
	char operationFileName[1024];
	char data[16];
	int file;
	int ret;
	sprintf(operationFileName,"%s%s%d/%s",SERVO_DRIVER_SYSFS_BASE,SERVO_DRIVER_SERVO_FILE,servoID,SERVO_DRIVER_SERVO_OFFSET_FILE);
	file=open(operationFileName,O_WRONLY);
	if(file==-1){
		fprintf(stderr,"MotionServer setServoOffset: failed to open %s",operationFileName);
		cleaning(errno);
	}
	/* Keep the value */
	servoOffsetBuffer[servoID]=value;
	sprintf(data,"%d",value);
	ret=write(file,data,strlen(data));
	if(ret==-1){
		fprintf(stderr,"MotionServer setServoOffset: failed to write in %s",operationFileName);
		cleaning(errno);
	}
	close(file);
}
void setServoMin(int servoID,int value){
	printf("setServoMin\n");
	char operationFileName[1024];
	char data[16];
	int file;
	int ret;
	sprintf(operationFileName,"%s%s%d/%s",SERVO_DRIVER_SYSFS_BASE,SERVO_DRIVER_SERVO_FILE,servoID,SERVO_DRIVER_SERVO_MIN_FILE);
	file=open(operationFileName,O_WRONLY);
	if(file==-1){
		fprintf(stderr,"MotionServer setServoMin: failed to open %s",operationFileName);
		cleaning(errno);
	}
	/* Keep the value */
	servoMinBuffer[servoID]=value;
	sprintf(data,"%d",value);
	ret=write(file,data,strlen(data));
	if(ret==-1){
		fprintf(stderr,"MotionServer setServoMin: failed to write in %s",operationFileName);
		cleaning(errno);
	}
	close(file);
}
void setServoMax(int servoID,int value){
	printf("setServoMax\n");
	char operationFileName[1024];
	char data[16];
	int file;
	int ret;
	sprintf(operationFileName,"%s%s%d/%s",SERVO_DRIVER_SYSFS_BASE,SERVO_DRIVER_SERVO_FILE,servoID,SERVO_DRIVER_SERVO_MAX_FILE);
	file=open(operationFileName,O_WRONLY);
	if(file==-1){
		fprintf(stderr,"MotionServer setServoMax: failed to open %s",operationFileName);
		cleaning(errno);
	}
	/* Keep the value */
	servoMaxBuffer[servoID]=value;
	sprintf(data,"%d",value);
	ret=write(file,data,strlen(data));
	if(ret==-1){
		fprintf(stderr,"MotionServer setServoMax: failed to write in %s",operationFileName);
		cleaning(errno);
	}
	close(file);
}
void getServoPosition(int servoID){
	sprintf(pBuffer,"%d",servoPositionBuffer[servoID]);
}
void getServoEnable(int servoID){
	sprintf(pBuffer,"%d",servoEnableBuffer[servoID]);
}
void getServoOffset(int servoID){
	sprintf(pBuffer,"%d",servoOffsetBuffer[servoID]);
}
void getServoMin(int servoID){
	sprintf(pBuffer,"%d",servoMinBuffer[servoID]);
}
void getServoMax(int servoID){
	sprintf(pBuffer,"%d",servoMaxBuffer[servoID]);
}

int writeDefaultConfig(char* file_path){
	char config_file_name[1024];
	char message_buffer[1024];
	int i;
	FILE *config_file;
	/* File path is not specified, using ~/.MotionServerrc */
	if((file_path==NULL) || (strcmp("",file_path)==0)){
		sprintf(config_file_name,"%s/.MotionServerrc",getenv("HOME"));
	} else {
		sprintf(config_file_name,"%s",file_path);
	}
	printf("Writing config file to: %s\n",config_file_name);
	config_file=fopen(config_file_name,"w");
	if (!config_file){
		sprintf(message_buffer,"Error in opening configuration file %s to write",config_file_name);
		perror(message_buffer);
		return(1);
	}
	fprintf(config_file,"[Module]\n");
	fprintf(config_file,"ModuleFilePath %s\n",moduleFilePathBuffer);
	fprintf(config_file,"ModuleLoad 1\n");
	fprintf(config_file,"[Default]\n");
	for(i=0;i<NBR_OF_MANAGED_SERVOS;i++){
		fprintf(config_file,"ServoPosition %d %d\n",i,servoPositionBuffer[i]);
		fprintf(config_file,"ServoOffset %d %d\n",i,servoOffsetBuffer[i]);
		fprintf(config_file,"ServoMin %d %d\n",i,servoMinBuffer[i]);
		fprintf(config_file,"ServoMax %d %d\n",i,servoMaxBuffer[i]);
		fprintf(config_file,"ServoEnable %d %d\n\n",i,servoEnableBuffer[i]);
	}
	fclose(config_file);
	return(0);
}
void cleaning(int err){
	/* Close the client socket*/
//	close(hSocket);
	/* Close the main server socket*/
//	close(hServerSocket);
//	exit(err);
}
